/*
 * This file is part of Sharenet.
 * 
 * Sharenet is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 * 
 * You may obtain a copy of the license at:
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 * 
 * Copyright (c) 2024 Continuist <continuist02@gmail.com>
 */

use sqlx::PgPool;
use uuid::Uuid;

use domain::{
    CreateProduct, CreateUser, Product, Result, UpdateProduct, UpdateUser, User,
};
use application::{Repository, Service};

#[derive(Clone)]
pub struct PostgresUserRepository {
    pool: PgPool,
}

impl PostgresUserRepository {
    pub fn new(pool: PgPool) -> Self {
        Self { pool }
    }
}

impl Repository<User> for PostgresUserRepository {
    async fn create(&self, data: CreateUser) -> Result<User> {
        let user = sqlx::query_as!(
            User,
            r#"
            INSERT INTO users (username, email)
            VALUES ($1, $2)
            RETURNING id, username, email, created_at, updated_at
            "#,
            data.username,
            data.email
        )
        .fetch_one(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        Ok(user)
    }

    async fn find_by_id(&self, id: Uuid) -> Result<User> {
        let user = sqlx::query_as!(
            User,
            r#"
            SELECT id, username, email, created_at, updated_at
            FROM users
            WHERE id = $1
            "#,
            id
        )
        .fetch_optional(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?
        .ok_or_else(|| domain::DomainError::NotFound(format!("User not found: {}", id)))?;

        Ok(user)
    }

    async fn find_all(&self) -> Result<Vec<User>> {
        let users = sqlx::query_as!(
            User,
            r#"
            SELECT id, username, email, created_at, updated_at
            FROM users
            "#
        )
        .fetch_all(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        Ok(users)
    }

    async fn update(&self, id: Uuid, data: UpdateUser) -> Result<User> {
        let user = sqlx::query_as!(
            User,
            r#"
            UPDATE users
            SET
                username = COALESCE($1, username),
                email = COALESCE($2, email),
                updated_at = CURRENT_TIMESTAMP
            WHERE id = $3
            RETURNING id, username, email, created_at, updated_at
            "#,
            data.username,
            data.email,
            id
        )
        .fetch_optional(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?
        .ok_or_else(|| domain::DomainError::NotFound(format!("User not found: {}", id)))?;

        Ok(user)
    }

    async fn delete(&self, id: Uuid) -> Result<()> {
        let result = sqlx::query!(
            r#"
            DELETE FROM users
            WHERE id = $1
            "#,
            id
        )
        .execute(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        if result.rows_affected() == 0 {
            return Err(domain::DomainError::NotFound(format!("User not found: {}", id)));
        }

        Ok(())
    }
}

#[derive(Clone)]
pub struct PostgresProductRepository {
    pool: PgPool,
}

impl PostgresProductRepository {
    pub fn new(pool: PgPool) -> Self {
        Self { pool }
    }
}

impl Repository<Product> for PostgresProductRepository {
    async fn create(&self, data: CreateProduct) -> Result<Product> {
        let product = sqlx::query_as!(
            Product,
            r#"
            INSERT INTO products (name, description)
            VALUES ($1, $2)
            RETURNING id, name, description, created_at, updated_at
            "#,
            data.name,
            data.description
        )
        .fetch_one(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        Ok(product)
    }

    async fn find_by_id(&self, id: Uuid) -> Result<Product> {
        let product = sqlx::query_as!(
            Product,
            r#"
            SELECT id, name, description, created_at, updated_at
            FROM products
            WHERE id = $1
            "#,
            id
        )
        .fetch_optional(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?
        .ok_or_else(|| domain::DomainError::NotFound(format!("Product not found: {}", id)))?;

        Ok(product)
    }

    async fn find_all(&self) -> Result<Vec<Product>> {
        let products = sqlx::query_as!(
            Product,
            r#"
            SELECT id, name, description, created_at, updated_at
            FROM products
            "#
        )
        .fetch_all(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        Ok(products)
    }

    async fn update(&self, id: Uuid, data: UpdateProduct) -> Result<Product> {
        let product = sqlx::query_as!(
            Product,
            r#"
            UPDATE products
            SET
                name = COALESCE($1, name),
                description = COALESCE($2, description),
                updated_at = CURRENT_TIMESTAMP
            WHERE id = $3
            RETURNING id, name, description, created_at, updated_at
            "#,
            data.name,
            data.description,
            id
        )
        .fetch_optional(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?
        .ok_or_else(|| domain::DomainError::NotFound(format!("Product not found: {}", id)))?;

        Ok(product)
    }

    async fn delete(&self, id: Uuid) -> Result<()> {
        let result = sqlx::query!(
            r#"
            DELETE FROM products
            WHERE id = $1
            "#,
            id
        )
        .execute(&self.pool)
        .await
        .map_err(|e| domain::DomainError::Internal(e.to_string()))?;

        if result.rows_affected() == 0 {
            return Err(domain::DomainError::NotFound(format!("Product not found: {}", id)));
        }

        Ok(())
    }
}

pub type PostgresUserService = Service<User, PostgresUserRepository>;
pub type PostgresProductService = Service<Product, PostgresProductRepository>;

#[cfg(test)]
mod tests {
    use super::*;
    use sqlx::PgPool;
    use sqlx::postgres::PgPoolOptions;
    use std::env;
    use chrono::Utc;

    // Test database setup
    async fn setup_test_db() -> PgPool {
        let database_url = env::var("DATABASE_URL")
            .unwrap_or_else(|_| "postgres://postgres:password@localhost:5432/sharenet_test".to_string());
        
        let pool = PgPoolOptions::new()
            .max_connections(5)
            .connect(&database_url)
            .await
            .expect("Failed to connect to test database");

        // Run migrations
        sqlx::migrate!("../../migrations")
            .run(&pool)
            .await
            .expect("Failed to run migrations");

        // Clean up any existing test data
        cleanup_test_data(&pool).await;

        pool
    }

    // Clean up test data
    async fn cleanup_test_data(pool: &PgPool) {
        sqlx::query("DELETE FROM products").execute(pool).await.ok();
        sqlx::query("DELETE FROM users").execute(pool).await.ok();
    }

    mod user_repository_tests {
        use super::*;

        #[tokio::test]
        async fn test_create_user() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let create_data = CreateUser {
                username: "testuser".to_string(),
                email: "test@example.com".to_string(),
            };

            let result = repo.create(create_data).await;
            assert!(result.is_ok());

            let user = result.unwrap();
            assert_eq!(user.username, "testuser");
            assert_eq!(user.email, "test@example.com");
            assert!(user.id != Uuid::nil());
            assert!(user.created_at <= Utc::now());
            assert!(user.updated_at <= Utc::now());

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_create_user_with_duplicate_username() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let create_data = CreateUser {
                username: "duplicate_user".to_string(),
                email: "test1@example.com".to_string(),
            };

            // Create first user
            let result1 = repo.create(create_data.clone()).await;
            assert!(result1.is_ok());

            // Try to create second user with same username
            let create_data2 = CreateUser {
                username: "duplicate_user".to_string(),
                email: "test2@example.com".to_string(),
            };
            let result2 = repo.create(create_data2).await;
            assert!(result2.is_err());

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_find_user_by_id() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let create_data = CreateUser {
                username: "finduser".to_string(),
                email: "find@example.com".to_string(),
            };

            let created_user = repo.create(create_data).await.unwrap();
            let found_user = repo.find_by_id(created_user.id).await;
            
            assert!(found_user.is_ok());
            let user = found_user.unwrap();
            assert_eq!(user.id, created_user.id);
            assert_eq!(user.username, "finduser");
            assert_eq!(user.email, "find@example.com");

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_find_user_by_nonexistent_id() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let nonexistent_id = Uuid::new_v4();
            let result = repo.find_by_id(nonexistent_id).await;
            
            assert!(result.is_err());
            match result.unwrap_err() {
                domain::DomainError::NotFound(msg) => {
                    assert!(msg.contains("User not found"));
                }
                _ => panic!("Expected NotFound error"),
            }

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_find_all_users() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            // Create multiple users
            let _user1 = repo.create(CreateUser {
                username: "user1".to_string(),
                email: "user1@example.com".to_string(),
            }).await.unwrap();

            let _user2 = repo.create(CreateUser {
                username: "user2".to_string(),
                email: "user2@example.com".to_string(),
            }).await.unwrap();

            let users = repo.find_all().await.unwrap();
            assert_eq!(users.len(), 2);
            
            let usernames: Vec<String> = users.iter().map(|u| u.username.clone()).collect();
            assert!(usernames.contains(&"user1".to_string()));
            assert!(usernames.contains(&"user2".to_string()));

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_find_all_users_empty() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let users = repo.find_all().await.unwrap();
            assert_eq!(users.len(), 0);

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_update_user() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let create_data = CreateUser {
                username: "updateuser".to_string(),
                email: "update@example.com".to_string(),
            };

            let user = repo.create(create_data).await.unwrap();
            let original_updated_at = user.updated_at;

            // Update username only
            let update_data = UpdateUser {
                username: Some("updateduser".to_string()),
                email: None,
            };

            let updated_user = repo.update(user.id, update_data).await.unwrap();
            assert_eq!(updated_user.username, "updateduser");
            assert_eq!(updated_user.email, "update@example.com"); // Should remain unchanged
            assert!(updated_user.updated_at > original_updated_at);

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_update_user_email_only() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let create_data = CreateUser {
                username: "emailuser".to_string(),
                email: "old@example.com".to_string(),
            };

            let user = repo.create(create_data).await.unwrap();

            let update_data = UpdateUser {
                username: None,
                email: Some("new@example.com".to_string()),
            };

            let updated_user = repo.update(user.id, update_data).await.unwrap();
            assert_eq!(updated_user.username, "emailuser"); // Should remain unchanged
            assert_eq!(updated_user.email, "new@example.com");

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_update_user_both_fields() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let create_data = CreateUser {
                username: "bothuser".to_string(),
                email: "both@example.com".to_string(),
            };

            let user = repo.create(create_data).await.unwrap();

            let update_data = UpdateUser {
                username: Some("newbothuser".to_string()),
                email: Some("newboth@example.com".to_string()),
            };

            let updated_user = repo.update(user.id, update_data).await.unwrap();
            assert_eq!(updated_user.username, "newbothuser");
            assert_eq!(updated_user.email, "newboth@example.com");

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_update_nonexistent_user() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let nonexistent_id = Uuid::new_v4();
            let update_data = UpdateUser {
                username: Some("nonexistent".to_string()),
                email: None,
            };

            let result = repo.update(nonexistent_id, update_data).await;
            assert!(result.is_err());
            match result.unwrap_err() {
                domain::DomainError::NotFound(msg) => {
                    assert!(msg.contains("User not found"));
                }
                _ => panic!("Expected NotFound error"),
            }

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_delete_user() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let create_data = CreateUser {
                username: "deleteuser".to_string(),
                email: "delete@example.com".to_string(),
            };

            let user = repo.create(create_data).await.unwrap();
            let user_id = user.id;

            // Verify user exists
            let found_user = repo.find_by_id(user_id).await;
            assert!(found_user.is_ok());

            // Delete user
            let delete_result = repo.delete(user_id).await;
            assert!(delete_result.is_ok());

            // Verify user no longer exists
            let found_user_after_delete = repo.find_by_id(user_id).await;
            assert!(found_user_after_delete.is_err());

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_delete_nonexistent_user() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let nonexistent_id = Uuid::new_v4();
            let result = repo.delete(nonexistent_id).await;
            
            assert!(result.is_err());
            match result.unwrap_err() {
                domain::DomainError::NotFound(msg) => {
                    assert!(msg.contains("User not found"));
                }
                _ => panic!("Expected NotFound error"),
            }

            cleanup_test_data(&pool).await;
        }
    }

    mod product_repository_tests {
        use super::*;

        #[tokio::test]
        async fn test_create_product() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            let create_data = CreateProduct {
                name: "Test Product".to_string(),
                description: "Test Description".to_string(),
            };

            let result = repo.create(create_data).await;
            assert!(result.is_ok());

            let product = result.unwrap();
            assert_eq!(product.name, "Test Product");
            assert_eq!(product.description, "Test Description");
            assert!(product.id != Uuid::nil());
            assert!(product.created_at <= Utc::now());
            assert!(product.updated_at <= Utc::now());

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_find_product_by_id() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            let create_data = CreateProduct {
                name: "Find Product".to_string(),
                description: "Find Description".to_string(),
            };

            let created_product = repo.create(create_data).await.unwrap();
            let found_product = repo.find_by_id(created_product.id).await;
            
            assert!(found_product.is_ok());
            let product = found_product.unwrap();
            assert_eq!(product.id, created_product.id);
            assert_eq!(product.name, "Find Product");
            assert_eq!(product.description, "Find Description");

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_find_product_by_nonexistent_id() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            let nonexistent_id = Uuid::new_v4();
            let result = repo.find_by_id(nonexistent_id).await;
            
            assert!(result.is_err());
            match result.unwrap_err() {
                domain::DomainError::NotFound(msg) => {
                    assert!(msg.contains("Product not found"));
                }
                _ => panic!("Expected NotFound error"),
            }

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_find_all_products() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            // Create multiple products
            let _product1 = repo.create(CreateProduct {
                name: "Product 1".to_string(),
                description: "Description 1".to_string(),
            }).await.unwrap();

            let _product2 = repo.create(CreateProduct {
                name: "Product 2".to_string(),
                description: "Description 2".to_string(),
            }).await.unwrap();

            let products = repo.find_all().await.unwrap();
            assert_eq!(products.len(), 2);
            
            let names: Vec<String> = products.iter().map(|p| p.name.clone()).collect();
            assert!(names.contains(&"Product 1".to_string()));
            assert!(names.contains(&"Product 2".to_string()));

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_find_all_products_empty() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            let products = repo.find_all().await.unwrap();
            assert_eq!(products.len(), 0);

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_update_product() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            let create_data = CreateProduct {
                name: "Update Product".to_string(),
                description: "Update Description".to_string(),
            };

            let product = repo.create(create_data).await.unwrap();
            let original_updated_at = product.updated_at;

            // Update name only
            let update_data = UpdateProduct {
                name: Some("Updated Product".to_string()),
                description: None,
            };

            let updated_product = repo.update(product.id, update_data).await.unwrap();
            assert_eq!(updated_product.name, "Updated Product");
            assert_eq!(updated_product.description, "Update Description"); // Should remain unchanged
            assert!(updated_product.updated_at > original_updated_at);

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_update_product_description_only() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            let create_data = CreateProduct {
                name: "Desc Product".to_string(),
                description: "Old Description".to_string(),
            };

            let product = repo.create(create_data).await.unwrap();

            let update_data = UpdateProduct {
                name: None,
                description: Some("New Description".to_string()),
            };

            let updated_product = repo.update(product.id, update_data).await.unwrap();
            assert_eq!(updated_product.name, "Desc Product"); // Should remain unchanged
            assert_eq!(updated_product.description, "New Description");

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_update_product_both_fields() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            let create_data = CreateProduct {
                name: "Both Product".to_string(),
                description: "Both Description".to_string(),
            };

            let product = repo.create(create_data).await.unwrap();

            let update_data = UpdateProduct {
                name: Some("New Both Product".to_string()),
                description: Some("New Both Description".to_string()),
            };

            let updated_product = repo.update(product.id, update_data).await.unwrap();
            assert_eq!(updated_product.name, "New Both Product");
            assert_eq!(updated_product.description, "New Both Description");

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_update_nonexistent_product() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            let nonexistent_id = Uuid::new_v4();
            let update_data = UpdateProduct {
                name: Some("Nonexistent Product".to_string()),
                description: None,
            };

            let result = repo.update(nonexistent_id, update_data).await;
            assert!(result.is_err());
            match result.unwrap_err() {
                domain::DomainError::NotFound(msg) => {
                    assert!(msg.contains("Product not found"));
                }
                _ => panic!("Expected NotFound error"),
            }

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_delete_product() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            let create_data = CreateProduct {
                name: "Delete Product".to_string(),
                description: "Delete Description".to_string(),
            };

            let product = repo.create(create_data).await.unwrap();
            let product_id = product.id;

            // Verify product exists
            let found_product = repo.find_by_id(product_id).await;
            assert!(found_product.is_ok());

            // Delete product
            let delete_result = repo.delete(product_id).await;
            assert!(delete_result.is_ok());

            // Verify product no longer exists
            let found_product_after_delete = repo.find_by_id(product_id).await;
            assert!(found_product_after_delete.is_err());

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_delete_nonexistent_product() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            
            let nonexistent_id = Uuid::new_v4();
            let result = repo.delete(nonexistent_id).await;
            
            assert!(result.is_err());
            match result.unwrap_err() {
                domain::DomainError::NotFound(msg) => {
                    assert!(msg.contains("Product not found"));
                }
                _ => panic!("Expected NotFound error"),
            }

            cleanup_test_data(&pool).await;
        }
    }

    mod service_tests {
        use super::*;
        use application::UseCase;

        #[tokio::test]
        async fn test_postgres_user_service() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            let service = PostgresUserService::new(repo);
            
            let create_data = CreateUser {
                username: "serviceuser".to_string(),
                email: "service@example.com".to_string(),
            };

            // Test create
            let user = service.create(create_data).await.unwrap();
            assert_eq!(user.username, "serviceuser");

            // Test get
            let found_user = service.get(user.id).await.unwrap();
            assert_eq!(found_user.id, user.id);

            // Test list
            let users = service.list().await.unwrap();
            assert_eq!(users.len(), 1);

            // Test update
            let update_data = UpdateUser {
                username: Some("updatedserviceuser".to_string()),
                email: None,
            };
            let updated_user = service.update(user.id, update_data).await.unwrap();
            assert_eq!(updated_user.username, "updatedserviceuser");

            // Test delete
            let delete_result = service.delete(user.id).await;
            assert!(delete_result.is_ok());

            cleanup_test_data(&pool).await;
        }

        #[tokio::test]
        async fn test_postgres_product_service() {
            let pool = setup_test_db().await;
            let repo = PostgresProductRepository::new(pool.clone());
            let service = PostgresProductService::new(repo);
            
            let create_data = CreateProduct {
                name: "Service Product".to_string(),
                description: "Service Description".to_string(),
            };

            // Test create
            let product = service.create(create_data).await.unwrap();
            assert_eq!(product.name, "Service Product");

            // Test get
            let found_product = service.get(product.id).await.unwrap();
            assert_eq!(found_product.id, product.id);

            // Test list
            let products = service.list().await.unwrap();
            assert_eq!(products.len(), 1);

            // Test update
            let update_data = UpdateProduct {
                name: Some("Updated Service Product".to_string()),
                description: None,
            };
            let updated_product = service.update(product.id, update_data).await.unwrap();
            assert_eq!(updated_product.name, "Updated Service Product");

            // Test delete
            let delete_result = service.delete(product.id).await;
            assert!(delete_result.is_ok());

            cleanup_test_data(&pool).await;
        }
    }

    mod error_handling_tests {
        use super::*;

        #[tokio::test]
        async fn test_database_connection_error() {
            // Test with invalid database URL
            let invalid_pool = PgPoolOptions::new()
                .max_connections(1)
                .connect("postgres://invalid:invalid@localhost:5432/nonexistent")
                .await;

            assert!(invalid_pool.is_err());
        }

        #[tokio::test]
        async fn test_concurrent_access() {
            let pool = setup_test_db().await;
            let repo = PostgresUserRepository::new(pool.clone());
            
            let create_data = CreateUser {
                username: "concurrentuser".to_string(),
                email: "concurrent@example.com".to_string(),
            };

            // Create a user
            let user = repo.create(create_data).await.unwrap();

            // Simulate concurrent reads
            let handles: Vec<_> = (0..10)
                .map(|_| {
                    let repo_clone = repo.clone();
                    let user_id = user.id;
                    tokio::spawn(async move {
                        repo_clone.find_by_id(user_id).await
                    })
                })
                .collect();

            let results = futures::future::join_all(handles).await;
            for result in results {
                assert!(result.unwrap().is_ok());
            }

            cleanup_test_data(&pool).await;
        }
    }
}
